@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2020 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013-2021 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,11 +27,6 @@
 
 #include <algorithm>
 
-extern "C"
-{
-#include <libxfce4util/libxfce4util.h>
-}
-
 using namespace WhiskerMenu;
 
 //-----------------------------------------------------------------------------
@@ -45,7 +40,11 @@ ApplicationsPage::ApplicationsPage(Window* window) :
 	garcon_set_environment_xdg(GARCON_ENVIRONMENT_XFCE);
 
 	const decltype(m_categories.size()) index = 0;
-	g_signal_connect_slot(get_button()->get_widget(), "toggled", &ApplicationsPage::show_category, this, index);
+	connect(get_button()->get_widget(), "toggled",
+		[this](GtkToggleButton* button)
+		{
+			show_category(button, index);
+		});
 }
 
 //-----------------------------------------------------------------------------
@@ -57,7 +56,7 @@ ApplicationsPage::~ApplicationsPage()
 
 //-----------------------------------------------------------------------------
 
-GtkTreeModel* ApplicationsPage::create_launcher_model(std::vector<std::string>& desktop_ids) const
+GtkTreeModel* ApplicationsPage::create_launcher_model(StringList& desktop_ids) const
 {
 	// Create new model for treeview
 	GtkListStore* store = gtk_list_store_new(
@@ -68,14 +67,15 @@ GtkTreeModel* ApplicationsPage::create_launcher_model(std::vector<std::string>&
 			G_TYPE_POINTER);
 
 	// Fetch menu items or remove them from list if missing
-	for (auto i = desktop_ids.begin(); i != desktop_ids.end(); ++i)
+	for (int i = 0; i < desktop_ids.size(); ++i)
 	{
-		if (i->empty())
+		const std::string& desktop_id = desktop_ids[i];
+		if (desktop_id.empty())
 		{
 			continue;
 		}
 
-		Launcher* launcher = find(*i);
+		Launcher* launcher = find(desktop_id);
 		if (launcher)
 		{
 			gtk_list_store_insert_with_values(
@@ -88,9 +88,8 @@ GtkTreeModel* ApplicationsPage::create_launcher_model(std::vector<std::string>&
 		}
 		else
 		{
-			i = desktop_ids.erase(i);
+			desktop_ids.erase(i);
 			--i;
-			wm_settings->set_modified();
 		}
 	}
 
@@ -170,9 +169,19 @@ bool ApplicationsPage::load()
 	clear();
 
 	// Load contents in thread if possible
-	GTask* task = g_task_new(nullptr, nullptr, &ApplicationsPage::load_contents_slot, this);
+	GTask* task = g_task_new(nullptr, nullptr,
+		+[](GObject*, GAsyncResult*, gpointer user_data)
+		{
+			static_cast<ApplicationsPage*>(user_data)->load_contents();
+		},
+		this);
 	g_task_set_task_data(task, this, nullptr);
-	g_task_run_in_thread(task, &ApplicationsPage::load_garcon_menu_slot);
+	g_task_run_in_thread(task,
+		+[](GTask* task, gpointer, gpointer task_data, GCancellable*)
+		{
+			static_cast<ApplicationsPage*>(task_data)->load_garcon_menu();
+			g_task_return_boolean(task, true);
+		});
 	g_object_unref(task);
 
 	return false;
@@ -235,7 +244,7 @@ void ApplicationsPage::load_garcon_menu()
 	}
 	else
 	{
-		m_garcon_menu = garcon_menu_new_for_path(wm_settings->custom_menu_file.c_str());
+		m_garcon_menu = garcon_menu_new_for_path(wm_settings->custom_menu_file);
 	}
 
 	// Load menu
@@ -250,8 +259,13 @@ void ApplicationsPage::load_garcon_menu()
 		return;
 	}
 
-	g_signal_connect_slot<GarconMenu*>(m_garcon_menu, "reload-required", &ApplicationsPage::invalidate, this);
-	load_menu(m_garcon_menu, nullptr, wm_settings->load_hierarchy);
+	connect(m_garcon_menu, "reload-required",
+		[this](GarconMenu*)
+		{
+			invalidate();
+		});
+
+	load_menu(m_garcon_menu, nullptr, wm_settings->view_mode == Settings::ViewAsTree);
 
 	// Create settings menu
 	gchar* path = xfce_resource_lookup(XFCE_RESOURCE_CONFIG, "menus/xfce-settings-manager.menu");
@@ -260,7 +274,11 @@ void ApplicationsPage::load_garcon_menu()
 
 	if (m_garcon_settings_menu)
 	{
-		g_signal_connect_slot<GarconMenu*>(m_garcon_settings_menu, "reload-required", &ApplicationsPage::invalidate, this);
+		connect(m_garcon_settings_menu, "reload-required",
+			[this](GarconMenu*)
+			{
+				invalidate();
+			});
 	}
 
 	// Load settings menu
@@ -272,14 +290,17 @@ void ApplicationsPage::load_garcon_menu()
 	}
 
 	// Sort items and categories
-	if (!wm_settings->load_hierarchy)
+	if (wm_settings->view_mode != Settings::ViewAsTree)
 	{
 		for (auto category : m_categories)
 		{
 			category->sort();
 		}
 	}
-	std::sort(m_categories.begin(), m_categories.end(), &Element::less_than);
+	if (wm_settings->sort_categories)
+	{
+		std::sort(m_categories.begin(), m_categories.end(), &Element::less_than);
+	}
 
 	// Create all items category
 	Category* category = new Category(nullptr);
@@ -311,7 +332,11 @@ void ApplicationsPage::load_contents()
 	for (decltype(m_categories.size()) i = 1; i < size; ++i)
 	{
 		CategoryButton* category_button = m_categories[i]->get_button();
-		g_signal_connect_slot(category_button->get_widget(), "toggled", &ApplicationsPage::show_category, this, i);
+		connect(category_button->get_widget(), "toggled",
+			[this, i](GtkToggleButton* button)
+			{
+				show_category(button, i);
+			});
 		category_buttons.push_back(category_button);
 	}
 
@@ -341,7 +366,11 @@ bool ApplicationsPage::load_menu(GarconMenu* menu, Category* parent_category, bo
 			GarconMenuItem* menuitem = GARCON_MENU_ITEM(li->data);
 
 			// Listen for changes
-			g_signal_connect_slot<GarconMenuItem*>(menuitem, "changed", &ApplicationsPage::invalidate, this);
+			connect(menuitem, "changed",
+				[this](GarconMenuItem*)
+				{
+					invalidate();
+				});
 
 			// Skip hidden items
 			if (!garcon_menu_element_get_visible(GARCON_MENU_ELEMENT(menuitem)))
