@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013, 2015, 2016 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013-2020 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,17 +61,17 @@ private:
 public:
 	const gchar* get_name() const
 	{
-		return NULL;
+		return nullptr;
 	}
 
 	const gchar* get_icon() const
 	{
-		return NULL;
+		return nullptr;
 	}
 
 	const gchar* get_command() const
 	{
-		return NULL;
+		return nullptr;
 	}
 
 #endif
@@ -83,15 +83,6 @@ public:
 	explicit Launcher(GarconMenuItem* item);
 	~Launcher();
 
-	enum
-	{
-		Type = 2
-	};
-	int get_type() const
-	{
-		return Type;
-	}
-
 	std::vector<DesktopAction*> get_actions() const
 	{
 		return m_actions;
@@ -117,11 +108,13 @@ public:
 		return garcon_menu_item_get_uri(m_item);
 	}
 
-	void run(GdkScreen* screen) const;
+	void hide();
+
+	void run(GdkScreen* screen) const override;
 
 	void run(GdkScreen* screen, DesktopAction* action) const;
 
-	guint search(const Query& query);
+	unsigned int search(const Query& query) override;
 
 	enum SearchFlag
 	{
@@ -137,7 +130,8 @@ private:
 	std::string m_search_generic_name;
 	std::string m_search_comment;
 	std::string m_search_command;
-	guint m_search_flags;
+	std::vector<std::string> m_search_keywords;
+	unsigned int m_search_flags;
 	std::vector<DesktopAction*> m_actions;
 };
 
