@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2020 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013-2021 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,6 +61,8 @@ Page::Page(Window* window, const gchar* icon, const gchar* text) :
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(m_widget), GTK_SHADOW_ETCHED_IN);
 	gtk_container_add(GTK_CONTAINER(m_widget), m_view->get_widget());
 	g_object_ref_sink(m_widget);
+
+	gtk_style_context_add_class(gtk_widget_get_style_context(m_widget), "launchers-pane");
 }
 
 //-----------------------------------------------------------------------------
@@ -111,11 +113,13 @@ void Page::select_first()
 
 void Page::update_view()
 {
-	if (dynamic_cast<LauncherIconView*>(m_view) && wm_settings->view_as_icons)
+	if ( ((wm_settings->view_mode == Settings::ViewAsIcons) && dynamic_cast<LauncherIconView*>(m_view))
+			|| ((wm_settings->view_mode != Settings::ViewAsIcons) && dynamic_cast<LauncherTreeView*>(m_view)) )
 	{
 		return;
 	}
 
+	g_assert(m_view);
 	LauncherView* view = m_view;
 	create_view();
 	m_view->set_model(view->get_model());
@@ -169,22 +173,55 @@ void Page::set_reorderable(bool reorderable)
 
 void Page::create_view()
 {
-	if (wm_settings->view_as_icons)
+	if (wm_settings->view_mode == Settings::ViewAsIcons)
 	{
 		m_view = new LauncherIconView();
-		g_signal_connect(m_view->get_widget(), "item-activated", G_CALLBACK(&Page::item_activated_slot), this);
+		connect(m_view->get_widget(), "item-activated",
+			[this](GtkIconView*, GtkTreePath* path)
+			{
+				launcher_activated(path);
+			});
 	}
 	else
 	{
 		m_view = new LauncherTreeView();
-		g_signal_connect(m_view->get_widget(), "row-activated", G_CALLBACK(&Page::row_activated_slot), this);
-		g_signal_connect_swapped(m_view->get_widget(), "start-interactive-search", G_CALLBACK(&gtk_widget_grab_focus), m_window->get_search_entry());
+		connect(m_view->get_widget(), "row-activated",
+			[this](GtkTreeView*, GtkTreePath* path, GtkTreeViewColumn*)
+			{
+				launcher_activated(path);
+			});
 	}
-	g_signal_connect_slot(m_view->get_widget(), "button-press-event", &Page::view_button_press_event, this);
-	g_signal_connect_slot(m_view->get_widget(), "button-release-event", &Page::view_button_release_event, this);
-	g_signal_connect_slot(m_view->get_widget(), "drag-data-get", &Page::view_drag_data_get, this);
-	g_signal_connect_slot(m_view->get_widget(), "drag-end", &Page::view_drag_end, this);
-	g_signal_connect_slot(m_view->get_widget(), "popup-menu", &Page::view_popup_menu_event, this);
+
+	connect(m_view->get_widget(), "button-press-event",
+		[this](GtkWidget*, GdkEvent* event) -> gboolean
+		{
+			return view_button_press_event(event);
+		});
+
+	connect(m_view->get_widget(), "button-release-event",
+		[this](GtkWidget*, GdkEvent* event) -> gboolean
+		{
+			return view_button_release_event(event);
+		});
+
+	connect(m_view->get_widget(), "drag-data-get",
+		[this](GtkWidget*, GdkDragContext*, GtkSelectionData* data, guint info, guint)
+		{
+			view_drag_data_get(data, info);
+		});
+
+	connect(m_view->get_widget(), "drag-end",
+		[this](GtkWidget*, GdkDragContext*)
+		{
+			view_drag_end();
+		});
+
+	connect(m_view->get_widget(), "popup-menu",
+		[this](GtkWidget*) -> gboolean
+		{
+			return view_popup_menu_event();
+		});
+
 	set_reorderable(m_reorderable);
 }
 
@@ -248,7 +285,7 @@ void Page::launcher_action_activated(GtkMenuItem* menuitem, DesktopAction* actio
 
 //-----------------------------------------------------------------------------
 
-gboolean Page::view_button_press_event(GtkWidget*, GdkEvent* event)
+gboolean Page::view_button_press_event(GdkEvent* event)
 {
 	GdkEventButton* button_event = reinterpret_cast<GdkEventButton*>(event);
 
@@ -296,7 +333,7 @@ gboolean Page::view_button_press_event(GtkWidget*, GdkEvent* event)
 
 //-----------------------------------------------------------------------------
 
-gboolean Page::view_button_release_event(GtkWidget*, GdkEvent* event)
+gboolean Page::view_button_release_event(GdkEvent* event)
 {
 	GdkEventButton* button_event = reinterpret_cast<GdkEventButton*>(event);
 	if (button_event->button != 1)
@@ -315,7 +352,7 @@ gboolean Page::view_button_release_event(GtkWidget*, GdkEvent* event)
 
 //-----------------------------------------------------------------------------
 
-void Page::view_drag_data_get(GtkWidget*, GdkDragContext*, GtkSelectionData* data, guint info, guint)
+void Page::view_drag_data_get(GtkSelectionData* data, guint info)
 {
 	if ((info != 1) || !m_selected_launcher)
 	{
@@ -334,7 +371,7 @@ void Page::view_drag_data_get(GtkWidget*, GdkDragContext*, GtkSelectionData* dat
 
 //-----------------------------------------------------------------------------
 
-void Page::view_drag_end(GtkWidget*, GdkDragContext*)
+void Page::view_drag_end()
 {
 	if (m_launcher_dragged)
 	{
@@ -345,7 +382,7 @@ void Page::view_drag_end(GtkWidget*, GdkDragContext*)
 
 //-----------------------------------------------------------------------------
 
-gboolean Page::view_popup_menu_event(GtkWidget*)
+gboolean Page::view_popup_menu_event()
 {
 	GtkTreePath* path = m_view->get_cursor();
 	if (!path)
@@ -376,7 +413,12 @@ void Page::create_context_menu(GtkTreePath* path, GdkEvent* event)
 
 	// Create context menu
 	GtkWidget* menu = gtk_menu_new();
-	g_signal_connect_slot(menu, "selection-done", &Page::destroy_context_menu, this);
+	connect(menu, "selection-done",
+		[this](GtkMenuShell* menu)
+		{
+			m_selected_launcher = nullptr;
+			gtk_widget_destroy(GTK_WIDGET(menu));
+		});
 
 	// Add menu items
 	GtkWidget* menuitem = gtk_menu_item_new_with_label(m_selected_launcher->get_display_name());
@@ -392,7 +434,11 @@ void Page::create_context_menu(GtkTreePath* path, GdkEvent* event)
 		for (auto action : actions)
 		{
 			menuitem = whiskermenu_image_menu_item_new(action->get_icon(), action->get_name());
-			g_signal_connect_slot(menuitem, "activate", &Page::launcher_action_activated, this, action);
+			connect(menuitem, "activate",
+				[this, action](GtkMenuItem* menuitem)
+				{
+					launcher_action_activated(menuitem, action);
+				});
 			gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 		}
 
@@ -403,33 +449,61 @@ void Page::create_context_menu(GtkTreePath* path, GdkEvent* event)
 	if (!m_window->get_favorites()->contains(m_selected_launcher))
 	{
 		menuitem = whiskermenu_image_menu_item_new("bookmark-new", _("Add to Favorites"));
-		g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &Page::add_selected_to_favorites, this);
+		connect(menuitem, "activate",
+			[this](GtkMenuItem*)
+			{
+				g_assert(m_selected_launcher);
+				m_window->get_favorites()->add(m_selected_launcher);
+			});
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 	}
 	else
 	{
 		menuitem = whiskermenu_image_menu_item_new("list-remove", _("Remove from Favorites"));
-		g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &Page::remove_selected_from_favorites, this);
+		connect(menuitem, "activate",
+			[this](GtkMenuItem*)
+			{
+				g_assert(m_selected_launcher);
+				m_window->get_favorites()->remove(m_selected_launcher);
+			});
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 	}
 
 	menuitem = whiskermenu_image_menu_item_new("list-add", _("Add to Desktop"));
-	g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &Page::add_selected_to_desktop, this);
+	connect(menuitem, "activate",
+		[this](GtkMenuItem*)
+		{
+			add_selected_to_desktop();
+		});
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
 	menuitem = whiskermenu_image_menu_item_new("list-add", _("Add to Panel"));
-	g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &Page::add_selected_to_panel, this);
+	connect(menuitem, "activate",
+		[this](GtkMenuItem*)
+		{
+			add_selected_to_panel();
+		});
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
 	menuitem = gtk_separator_menu_item_new();
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
 	menuitem = whiskermenu_image_menu_item_new("gtk-edit", _("Edit Application..."));
-	g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &Page::edit_selected, this);
+	connect(menuitem, "activate",
+		[this](GtkMenuItem*)
+		{
+			edit_selected();
+		});
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
 	menuitem = whiskermenu_image_menu_item_new("edit-delete", _("Hide Application"));
-	g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &Page::hide_selected, this);
+	connect(menuitem, "activate",
+		[this](GtkMenuItem*)
+		{
+			g_assert(m_selected_launcher);
+			m_window->hide();
+			m_selected_launcher->hide();
+		});
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
 	extend_context_menu(menu);
@@ -448,15 +522,6 @@ void Page::create_context_menu(GtkTreePath* path, GdkEvent* event)
 
 //-----------------------------------------------------------------------------
 
-void Page::destroy_context_menu(GtkMenuShell* menu)
-{
-	m_selected_launcher = nullptr;
-
-	gtk_widget_destroy(GTK_WIDGET(menu));
-}
-
-//-----------------------------------------------------------------------------
-
 void Page::extend_context_menu(GtkWidget*)
 {
 }
@@ -543,14 +608,6 @@ void Page::add_selected_to_panel()
 
 //-----------------------------------------------------------------------------
 
-void Page::add_selected_to_favorites()
-{
-	g_assert(m_selected_launcher);
-	m_window->get_favorites()->add(m_selected_launcher);
-}
-
-//-----------------------------------------------------------------------------
-
 void Page::edit_selected()
 {
 	g_assert(m_selected_launcher);
@@ -571,22 +628,3 @@ void Page::edit_selected()
 }
 
 //-----------------------------------------------------------------------------
-
-void Page::hide_selected()
-{
-	g_assert(m_selected_launcher);
-
-	m_window->hide();
-
-	m_selected_launcher->hide();
-}
-
-//-----------------------------------------------------------------------------
-
-void Page::remove_selected_from_favorites()
-{
-	g_assert(m_selected_launcher);
-	m_window->get_favorites()->remove(m_selected_launcher);
-}
-
-//-----------------------------------------------------------------------------
