@@ -24,16 +24,10 @@
 #include "slot.h"
 #include "window.h"
 
-extern "C"
-{
-#include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
-}
 
 using namespace WhiskerMenu;
 
-bool Plugin::m_menu_shown = false;
-
 //-----------------------------------------------------------------------------
 
 extern "C" void whiskermenu_construct(XfcePanelPlugin* plugin)
@@ -42,40 +36,6 @@ extern "C" void whiskermenu_construct(XfcePanelPlugin* plugin)
 	new Plugin(plugin);
 }
 
-static void plugin_free(XfcePanelPlugin*, gpointer user_data)
-{
-	delete static_cast<Plugin*>(user_data);
-}
-
-// Wait for grab; allows modifier as shortcut
-// Adapted from http://git.xfce.org/xfce/xfce4-panel/tree/common/panel-utils.c#n122
-static bool can_grab(GtkWidget* widget)
-{
-	GdkWindow* window = gtk_widget_get_window(widget);
-	GdkDisplay* display = gdk_window_get_display(window);
-	GdkSeat* seat = gdk_display_get_default_seat(display);
-
-	for (int i = 0; i < 5; ++i)
-	{
-		const GdkGrabStatus grab_status = gdk_seat_grab(seat,
-				window,
-				GDK_SEAT_CAPABILITY_ALL,
-				false,
-				nullptr,
-				nullptr,
-				nullptr,
-				nullptr);
-		if (grab_status == GDK_GRAB_SUCCESS)
-		{
-			gdk_seat_ungrab(seat);
-			return true;
-		}
-		g_usleep(100000);
-	}
-
-	return false;
-}
-
 #if !LIBXFCE4PANEL_CHECK_VERSION(4,13,0)
 static void widget_add_css(GtkWidget* widget, const gchar* css)
 {
@@ -94,14 +54,11 @@ Plugin::Plugin(XfcePanelPlugin* plugin) :
 	m_plugin(plugin),
 	m_window(nullptr),
 	m_opacity(100),
-	m_file_icon(false)
+	m_file_icon(false),
+	m_menu_shown(false)
 {
 	// Load settings
 	wm_settings = new Settings;
-	for (int i = Settings::CommandSwitchUser; i < Settings::CommandLogOut; ++i)
-	{
-		wm_settings->command[i]->set_shown(false);
-	}
 	wm_settings->load(xfce_resource_lookup(XFCE_RESOURCE_CONFIG, "xfce4/whiskermenu/defaults.rc"));
 	wm_settings->m_button_title_default = wm_settings->button_title;
 	wm_settings->load(xfce_panel_plugin_lookup_rc_file(m_plugin));
@@ -120,13 +77,24 @@ Plugin::Plugin(XfcePanelPlugin* plugin) :
 		}
 	}
 
+	// Switch to new icon only if theme is missing old icon
+	if ((wm_settings->button_icon_name == "xfce4-whiskermenu")
+			&& !gtk_icon_theme_has_icon(gtk_icon_theme_get_default(), "xfce4-whiskermenu"))
+	{
+		wm_settings->button_icon_name = "org.xfce.panel.whiskermenu";
+	}
+
 	// Create toggle button
 	m_button = xfce_panel_create_toggle_button();
 	gtk_widget_set_name(m_button, "whiskermenu-button");
 #if !LIBXFCE4PANEL_CHECK_VERSION(4,13,0)
 	widget_add_css(m_button, ".xfce4-panel button { padding: 1px; }");
 #endif
-	g_signal_connect_slot(m_button, "toggled", &Plugin::button_toggled, this);
+	connect(m_button, "toggled",
+		[this](GtkToggleButton* button)
+		{
+			button_toggled(button);
+		});
 	gtk_widget_show(m_button);
 
 	m_button_box = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2));
@@ -135,8 +103,8 @@ Plugin::Plugin(XfcePanelPlugin* plugin) :
 	gtk_widget_show(GTK_WIDGET(m_button_box));
 
 	m_button_icon = GTK_IMAGE(gtk_image_new());
-	icon_changed(wm_settings->button_icon_name.c_str());
-	gtk_widget_set_tooltip_markup(m_button, wm_settings->button_title.c_str());
+	icon_changed(wm_settings->button_icon_name);
+	gtk_widget_set_tooltip_markup(m_button, wm_settings->button_title);
 	gtk_box_pack_start(m_button_box, GTK_WIDGET(m_button_icon), true, false, 0);
 	if (wm_settings->button_icon_visible)
 	{
@@ -149,7 +117,7 @@ Plugin::Plugin(XfcePanelPlugin* plugin) :
 	gtk_widget_set_sensitive(GTK_WIDGET(m_button_icon), false);
 
 	m_button_label = GTK_LABEL(gtk_label_new(nullptr));
-	gtk_label_set_markup(m_button_label, wm_settings->button_title.c_str());
+	gtk_label_set_markup(m_button_label, wm_settings->button_title);
 	gtk_box_pack_start(m_button_box, GTK_WIDGET(m_button_label), true, true, 0);
 	if (wm_settings->button_title_visible)
 	{
@@ -162,34 +130,62 @@ Plugin::Plugin(XfcePanelPlugin* plugin) :
 	xfce_panel_plugin_add_action_widget(plugin, m_button);
 
 	// Connect plugin signals to functions
-	g_signal_connect(plugin, "free-data", G_CALLBACK(&plugin_free), this);
-	g_signal_connect_slot<XfcePanelPlugin*>(plugin, "configure-plugin", &Plugin::configure, this);
-	g_signal_connect_slot(plugin, "mode-changed", &Plugin::mode_changed, this);
-	g_signal_connect_slot(plugin, "remote-event", &Plugin::remote_event, this);
-	g_signal_connect_slot<XfcePanelPlugin*>(plugin, "save", &Plugin::save, this);
-	g_signal_connect_slot<XfcePanelPlugin*>(plugin, "about", &Plugin::show_about, this);
-	g_signal_connect_slot(plugin, "size-changed", &Plugin::size_changed, this);
+	connect(m_plugin, "free-data",
+		[this](XfcePanelPlugin*)
+		{
+			delete this;
+		});
+
+	connect(m_plugin, "configure-plugin",
+		[this](XfcePanelPlugin*)
+		{
+			configure();
+		});
+
+	connect(m_plugin, "mode-changed",
+		[this](XfcePanelPlugin*, XfcePanelPluginMode mode)
+		{
+			mode_changed(mode);
+		});
+
+	connect(m_plugin, "remote-event",
+		[this](XfcePanelPlugin*, const gchar* name, const GValue* value) -> gboolean
+		{
+			return remote_event(name, value);
+		});
+
+	connect(m_plugin, "save",
+		[this](XfcePanelPlugin*)
+		{
+			save();
+		});
+
+	connect(m_plugin, "about",
+		[this](XfcePanelPlugin*)
+		{
+			show_about();
+		});
+
+	connect(m_plugin, "size-changed",
+		[this](XfcePanelPlugin*, gint size) -> gboolean
+		{
+			return size_changed(size);
+		});
 
 	xfce_panel_plugin_menu_show_about(plugin);
 	xfce_panel_plugin_menu_show_configure(plugin);
 	xfce_panel_plugin_menu_insert_item(plugin, GTK_MENU_ITEM(wm_settings->command[Settings::CommandMenuEditor]->get_menuitem()));
 
-	mode_changed(m_plugin, xfce_panel_plugin_get_mode(m_plugin));
-
-	g_signal_connect_slot<GtkWidget*,GtkStyle*>(m_button, "style-set", &Plugin::update_size, this);
-	g_signal_connect_slot<GtkWidget*,GdkScreen*>(m_button, "screen-changed", &Plugin::update_size, this);
+	mode_changed(xfce_panel_plugin_get_mode(m_plugin));
 
 	// Create menu window
 	m_window = new Window(this);
-	g_signal_connect_slot<GtkWidget*>(m_window->get_widget(), "unmap", &Plugin::menu_hidden, this);
 }
 
 //-----------------------------------------------------------------------------
 
 Plugin::~Plugin()
 {
-	save();
-
 	delete m_window;
 	m_window = nullptr;
 
@@ -208,13 +204,6 @@ Plugin::ButtonStyle Plugin::get_button_style() const
 
 //-----------------------------------------------------------------------------
 
-std::string Plugin::get_button_title() const
-{
-	return wm_settings->button_title;
-}
-
-//-----------------------------------------------------------------------------
-
 std::string Plugin::get_button_title_default()
 {
 	return wm_settings->m_button_title_default;
@@ -222,9 +211,13 @@ std::string Plugin::get_button_title_default()
 
 //-----------------------------------------------------------------------------
 
-std::string Plugin::get_button_icon_name() const
+void Plugin::menu_hidden(bool lost_focus)
 {
-	return wm_settings->button_icon_name;
+	if (!lost_focus)
+	{
+		m_menu_shown = false;
+	}
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_button), false);
 }
 
 //-----------------------------------------------------------------------------
@@ -261,9 +254,7 @@ void Plugin::set_button_style(ButtonStyle style)
 		gtk_widget_set_has_tooltip(m_button, true);
 	}
 
-	wm_settings->set_modified();
-
-	size_changed(m_plugin, xfce_panel_plugin_get_size(m_plugin));
+	update_size();
 }
 
 //-----------------------------------------------------------------------------
@@ -271,11 +262,10 @@ void Plugin::set_button_style(ButtonStyle style)
 void Plugin::set_button_title(const std::string& title)
 {
 	wm_settings->button_title = title;
-	wm_settings->set_modified();
-	gtk_label_set_markup(m_button_label, wm_settings->button_title.c_str());
-	gtk_widget_set_tooltip_markup(m_button, wm_settings->button_title.c_str());
+	gtk_label_set_markup(m_button_label, wm_settings->button_title);
+	gtk_widget_set_tooltip_markup(m_button, wm_settings->button_title);
 	gtk_widget_set_has_tooltip(m_button, !wm_settings->button_title_visible);
-	size_changed(m_plugin, xfce_panel_plugin_get_size(m_plugin));
+	update_size();
 }
 
 //-----------------------------------------------------------------------------
@@ -283,9 +273,8 @@ void Plugin::set_button_title(const std::string& title)
 void Plugin::set_button_icon_name(const std::string& icon)
 {
 	wm_settings->button_icon_name = icon;
-	wm_settings->set_modified();
 	icon_changed(icon.c_str());
-	size_changed(m_plugin, xfce_panel_plugin_get_size(m_plugin));
+	update_size();
 }
 
 //-----------------------------------------------------------------------------
@@ -318,7 +307,6 @@ void Plugin::button_toggled(GtkToggleButton* button)
 	{
 		if (gtk_widget_get_visible(m_window->get_widget()))
 		{
-			m_menu_shown = false;
 			m_window->hide();
 		}
 		xfce_panel_plugin_block_autohide(m_plugin, false);
@@ -332,18 +320,15 @@ void Plugin::button_toggled(GtkToggleButton* button)
 
 //-----------------------------------------------------------------------------
 
-void Plugin::menu_hidden()
-{
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_button), false);
-	save();
-}
-
-//-----------------------------------------------------------------------------
-
 void Plugin::configure()
 {
 	SettingsDialog* dialog = new SettingsDialog(this);
-	g_signal_connect_slot<GtkWidget*>(dialog->get_widget(), "destroy", &Plugin::save, this);
+	connect(dialog->get_widget(), "destroy",
+		[this, dialog](GtkWidget*)
+		{
+			save();
+			delete dialog;
+		});
 }
 
 //-----------------------------------------------------------------------------
@@ -364,7 +349,7 @@ void Plugin::icon_changed(const gchar* icon)
 
 //-----------------------------------------------------------------------------
 
-void Plugin::mode_changed(XfcePanelPlugin*, XfcePanelPluginMode mode)
+void Plugin::mode_changed(XfcePanelPluginMode mode)
 {
 	gtk_label_set_angle(m_button_label, (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL) ? 270: 0);
 	update_size();
@@ -372,7 +357,7 @@ void Plugin::mode_changed(XfcePanelPlugin*, XfcePanelPluginMode mode)
 
 //-----------------------------------------------------------------------------
 
-gboolean Plugin::remote_event(XfcePanelPlugin*, gchar* name, GValue* value)
+gboolean Plugin::remote_event(const gchar* name, const GValue* value)
 {
 	if (strcmp(name, "popup"))
 	{
@@ -391,10 +376,6 @@ gboolean Plugin::remote_event(XfcePanelPlugin*, gchar* name, GValue* value)
 	{
 		m_window->hide();
 	}
-	else if (!can_grab(gtk_widget_get_toplevel(m_button)))
-	{
-		g_printerr("xfce4-whiskermenu-plugin: Unable to get keyboard. Menu popup failed.\n");
-	}
 	else if (value && G_VALUE_HOLDS_BOOLEAN(value) && g_value_get_boolean(value))
 	{
 		show_menu(true);
@@ -411,8 +392,6 @@ gboolean Plugin::remote_event(XfcePanelPlugin*, gchar* name, GValue* value)
 
 void Plugin::save()
 {
-	m_window->save();
-
 	if (wm_settings->get_modified())
 	{
 		wm_settings->save(xfce_panel_plugin_save_location(m_plugin, true));
@@ -433,7 +412,7 @@ void Plugin::show_about()
 		"comments", _("Alternate application launcher for Xfce"),
 		"copyright", _("Copyright \302\251 2013-2021 Graeme Gott"),
 		"license", XFCE_LICENSE_GPL,
-		"logo-icon-name", "xfce4-whiskermenu",
+		"logo-icon-name", "org.xfce.panel.whiskermenu",
 		"program-name", PACKAGE_NAME,
 		"translator-credits", _("translator-credits"),
 		"version", PACKAGE_VERSION,
@@ -443,7 +422,7 @@ void Plugin::show_about()
 
 //-----------------------------------------------------------------------------
 
-gboolean Plugin::size_changed(XfcePanelPlugin*, gint size)
+gboolean Plugin::size_changed(gint size)
 {
 	GtkOrientation panel_orientation = xfce_panel_plugin_get_orientation(m_plugin);
 	GtkOrientation orientation = panel_orientation;
@@ -493,7 +472,7 @@ gboolean Plugin::size_changed(XfcePanelPlugin*, gint size)
 			max_height *= 6;
 		}
 
-		GdkPixbuf* pixbuf = gdk_pixbuf_new_from_file_at_size(wm_settings->button_icon_name.c_str(), max_width, max_height, nullptr);
+		GdkPixbuf* pixbuf = gdk_pixbuf_new_from_file_at_size(wm_settings->button_icon_name, max_width, max_height, nullptr);
 		if (pixbuf)
 		{
 			// Handle high dpi
@@ -554,7 +533,7 @@ gboolean Plugin::size_changed(XfcePanelPlugin*, gint size)
 
 void Plugin::update_size()
 {
-	size_changed(m_plugin, xfce_panel_plugin_get_size(m_plugin));
+	size_changed(xfce_panel_plugin_get_size(m_plugin));
 }
 
 //-----------------------------------------------------------------------------
@@ -567,7 +546,6 @@ void Plugin::show_menu(bool at_cursor)
 		{
 			delete m_window;
 			m_window = new Window(this);
-			g_signal_connect_slot<GtkWidget*>(m_window->get_widget(), "unmap", &Plugin::menu_hidden, this);
 		}
 		m_opacity = wm_settings->menu_opacity;
 	}
