@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2020 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013-2021 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,8 +36,7 @@ public:
 	void unset_menu_items();
 
 private:
-	void extend_context_menu(GtkWidget* menu);
-	void clear_menu();
+	void extend_context_menu(GtkWidget* menu) override;
 };
 
 }
